import processing.serial.*;

Serial arduinoPort;  // Create object from Serial class
String arduinoPortName; // name of the port
int arduinoPortListLength;

void setup_serial_connection() {
  printArray(Serial.list());
  arduinoPortListLength = Serial.list().length;
  if (arduinoPortListLength>0) {
    arduinoPortName = Serial.list()[Serial.list().length-1];
    println("opening port: "+arduinoPortName);
    arduinoPort = new Serial(this, arduinoPortName, 9600);
  }
}

void check_serial_health() {
  int l = Serial.list().length;
  if (l<arduinoPortListLength) {
    arduinoPortListLength=l;
    arduinoPort.stop();
    print("disconnected!");
  }
  if (l>arduinoPortListLength) {
    print("reconnected!");
    delay(200);
    setup_serial_connection();
  }
}
class TimeStampedString {
  String string;
  int millis;
  TimeStampedString(String s) {
    string = s;
    millis = millis();
  }
}
ArrayList<TimeStampedString> arduinoStack = new ArrayList<TimeStampedString>();
Object arduinoStackLock = new Object();

Boolean arduinoMessageAvailable() {
  synchronized(arduinoStackLock) {
    return arduinoStack.size()>0;
  }
}
TimeStampedString getArduinoMessage() {
  synchronized(arduinoStackLock) {
    if (arduinoStack.size()==0) return null;
    return arduinoStack.remove(0);
  }
}

String arduinoMsg="";

void serialEvent(Serial myPort) {
  while( myPort.available() > 0) {  // If data is available,
    int val = myPort.read();    // read it and store it in val
    if ( (val==13) || (val==10) ) { // end of line?
      if (arduinoMsg.length()>0) {
        synchronized(arduinoStackLock) {
          TimeStampedString tss = new TimeStampedString(arduinoMsg);
          arduinoStack.add(tss);
          arduinoMsg="";
        }
      }
    } else {
      byte[] asciibytes = { (byte)val };
      try {
      arduinoMsg = arduinoMsg + new String(asciibytes, "ASCII");; //char(val); // add character to buffer
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }
}
